Language: [English](README.md) | [中文](README_zh-CN.md)

# simple_update
![Pub Version](https://img.shields.io/pub/v/simple_update?style=flat-square)
![Platform](https://img.shields.io/badge/platform-flutter%20android-brightgreen)

APP更新的最简单方式（下载并触发安装），暂时仅支持Android

默认情况下，会从[avenge.cn](https://avenge.cn)下载最新版本的APP，这是一个简单的APP版本管理系统，欢迎试用^_^

你也可以设置自己的服务器来发布APP新版本。

## 开始

1. 注册账号

   [https://avenge.cn/register](https://avenge.cn/register)
2. 创建应用与版本

   [https://avenge.cn/home/resources/apps/new](https://avenge.cn/home/resources/apps/new)
   [https://avenge.cn/home/resources/versions/new](https://avenge.cn/home/resources/versions/new)


3. 安装
   ```yaml
   dependencies:
       simple_update: ^1.0.0
   ```
   
    **Android**
  
    在 AndroidManifest.xml 的 ```<application>``` 节点中添加引用.
    ```xml
    <provider
        android:name="sk.fourq.otaupdate.OtaUpdateFileProvider"
        android:authorities="${applicationId}.ota_update_provider"
        android:exported="false"
        android:grantUriPermissions="true">
        <meta-data
            android:name="android.support.FILE_PROVIDER_PATHS"
            android:resource="@xml/filepaths" />
    </provider>
    ```
    详情请看 [AndroidManifest.xml](example/android/app/src/main/AndroidManifest.xml)
  
    然后，创建文件 ```android/src/main/res/xml/filepaths.xml``` ，填入以下内容。
    
    这将允许插件下载新版本APP文件到 Download 文件夹。
  
    ```xml
    <?xml version="1.0" encoding="utf-8"?>
    <paths xmlns:android="http://schemas.android.com/apk/res/android">
        <external-path name="external_download" path="Download"/>
    </paths>
    ```
  
    详情请看 [filepaths.xml](example/android/app/src/main/res/xml/filepaths.xml)
  
    **注意**
    Google Play Protect 可能会引发安装失败的问题。

4. 用法
   ```dart
   import 'package:simple_update/simple_update.dart';
  
   ... 
   RaisedButton(
      onPressed: () async {
        var updater = new SimpleUpdate(
          appId: 1,
          appKey: 'g4rehwe8fq4qe9rgh123',
          apiPrefix: 'http://avenge.cn/api');
   
        var res = await updater.checkUpdate(Platform.Android);
        if (res == true) {
          var re = await showDialog<bool>(
            context: context,
            builder: (context) => AlertDialog(
              title: Text('New version'),
              content: Text('Install the new version?'),
              actions: [
                FlatButton(
                    onPressed: () => Navigator.pop(context),
                    child: Text('Cancel')),
                FlatButton(
                    onPressed: () => Navigator.pop(context, true),
                    child: Text('Yes')),
              ],
            ),
          );
          if (re == true) {
            var r = await updater.update(Platform.Android);
            if (r is Stream<Event>) {
              r.listen((event) {
                switch (event.status) {
                  case Status.DOWNLOADING:
                    print('downloading %${event.value}');
                    break;
                  case Status.INSTALLING:
                    print('installing');
                    break;
                  case Status.ALREADY_RUNNING_ERROR:
                    print('download is already running');
                    break;
                  case Status.PERMISSION_NOT_GRANTED_ERROR:
                    print(
                        'could not continue because of missing permissions');
                    break;
                  case Status.INTERNAL_ERROR:
                  case Status.DOWNLOAD_ERROR:
                  case Status.CHECKSUM_ERROR:
                    print('error ${event.value}');
                }
              });
            }
          }
        } else {
          print('no latest version');
        }
      },
      child: Text('Update'),
   )
   ...
   ```

## 构建版本发布服务器

  
   [SimpleUpdate.apiPrefix] 的默认值是 'https://avenge.cn/api', 你可以指定自己的服务器来发布APP新版本.
   ```dart
   var updater = SimpleUpdate(apiPrefix: 'your own server');
   ```
  
  当获取新版本时，会向服务器发送一个get请求，携带如下参数
  > https://avenge.cn/api/latest?app_id=456&app_key=abc123&platform=0

  platform参数: 0:android 1:ios 2:fuchsia 3:linux 4:windows 5:macOS

  然后，服务器会返回json
  ```json
  {
    "code": 0,
    "message": "success",
    "data": {
     "id": 123,
     "app_id": 456,
     "platform": 0,
     "name": "1.0.1",
     "number": 24,
     "src": "https://avenge.cn/storage/apps/2020-07/app_name_1.0.1+24.apk",
     "sha256": "sha256 of file",
     "created_at": "2020-07-29 12:15:46"
    }
  }
  ```
  如果 *code* 值是 0 ，意味着提交成功。

  如果发生了错误，将会是：
  ```json
  {
    "code": -1,
    "message": "something wrong ..."
  }
  ```

## 全部项目
| 插件                                                     | 状态                                                       | 描述                                                  |
| ------------------------------------------------------------ | ------------------------------------------------------------ | ------------------------------------------------------------ |
| [simple_log](https://github.com/creatint/flutter_simple_log) | ![Pub Version](https://img.shields.io/pub/v/simple_log?style=flat-square) | 最简单的日志收集方式，支持所有平台 |
| [simple_update](https://github.com/creatint/flutter_simple_update) | ![Pub Version](https://img.shields.io/pub/v/simple_upate?style=flat-square) | 最简单的应用更新方式，暂时仅支持Android |
