import 'dart:async';

import 'package:flutter/material.dart';
import 'package:simple_update/simple_update.dart';
import 'package:flutter/services.dart';

class Home extends StatefulWidget {
  @override
  _HomeState createState() => _HomeState();
}

class _HomeState extends State<Home> {
  String _platformVersion = 'Unknown';
  SimpleUpdate updater;
  bool downloading = false;

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await SimpleUpdate.platformVersion;
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  void initState() {
    super.initState();
    initPlatformState();
    updater = new SimpleUpdate(
        appId: 1,
        appKey: 'g4rehwe8fq4qe9rgh4q',
        apiPrefix: 'http://192.168.101.43:8012/api');
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('SimpleUpdate'),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Text('Running on: $_platformVersion\n'),
            RaisedButton(
              onPressed: () async {
                var res = await updater.checkUpdate(Platform.Android);
                if (res == true) {
                  var re = await showDialog<bool>(
                    context: context,
                    builder: (context) => AlertDialog(
                      title: Text('New version'),
                      content: Text('Install the new version?'),
                      actions: [
                        FlatButton(
                            onPressed: () => Navigator.pop(context),
                            child: Text('Cancel')),
                        FlatButton(
                            onPressed: () => Navigator.pop(context, true),
                            child: Text('Yes')),
                      ],
                    ),
                  );
                  if (re == true) {
                    var r = await updater.update(Platform.Android);
                    if (r is Stream<Event>) {
                      r.listen((event) {
                        switch (event.status) {
                          case Status.DOWNLOADING:
                            print('downloading %${event.value}');
                            break;
                          case Status.INSTALLING:
                            print('installing');
                            break;
                          case Status.ALREADY_RUNNING_ERROR:
                            print('download is already running');
                            break;
                          case Status.PERMISSION_NOT_GRANTED_ERROR:
                            print(
                                'could not continue because of missing permissions');
                            break;
                          case Status.INTERNAL_ERROR:
                          case Status.DOWNLOAD_ERROR:
                          case Status.CHECKSUM_ERROR:
                            print('error ${event.value}');
                        }
                      });
                    }
                  }
                } else {
                  print('no latest version');
                }
              },
              child: Text('Update'),
            )
          ],
        ),
      ),
    );
  }
}
